<?php

namespace B2buy\Service\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use B2buy\Service\Http\Requests\QuoteRequest;
use B2buy\Service\Interfaces\QuoteRepositoryInterface;
use B2buy\Service\Models\Quote;

/**
 * Resource controller class for quote.
 */
class QuoteResourceController extends BaseController
{

    /**
     * Initialize quote resource controller.
     *
     * @param type QuoteRepositoryInterface $quote
     *
     * @return null
     */
    public function __construct(QuoteRepositoryInterface $quote)
    {
        parent::__construct();
        $this->repository = $quote;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\B2buy\Service\Repositories\Criteria\QuoteResourceCriteria::class);
    }

    /**
     * Display a list of quote.
     *
     * @return Response
     */
    public function index(QuoteRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\B2buy\Service\Repositories\Presenter\QuotePresenter::class)
                ->$function();
        }

        $quotes = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('service::quote.names'))
            ->view('service::quote.index', true)
            ->data(compact('quotes', 'view'))
            ->output();
    }

    /**
     * Display quote.
     *
     * @param Request $request
     * @param Model   $quote
     *
     * @return Response
     */
    public function show(QuoteRequest $request, Quote $quote)
    {

        if ($quote->exists) {
            $view = 'service::quote.show';
        } else {
            $view = 'service::quote.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('service::quote.name'))
            ->data(compact('quote'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new quote.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(QuoteRequest $request)
    {

        $quote = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('service::quote.name')) 
            ->view('service::quote.create', true) 
            ->data(compact('quote'))
            ->output();
    }

    /**
     * Create new quote.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(QuoteRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $quote                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('service::quote.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('service/quote/' . $quote->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/service/quote'))
                ->redirect();
        }

    }

    /**
     * Show quote for editing.
     *
     * @param Request $request
     * @param Model   $quote
     *
     * @return Response
     */
    public function edit(QuoteRequest $request, Quote $quote)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('service::quote.name'))
            ->view('service::quote.edit', true)
            ->data(compact('quote'))
            ->output();
    }

    /**
     * Update the quote.
     *
     * @param Request $request
     * @param Model   $quote
     *
     * @return Response
     */
    public function update(QuoteRequest $request, Quote $quote)
    {
        try {
            $attributes = $request->all();

            $quote->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('service::quote.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('service/quote/' . $quote->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('service/quote/' . $quote->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the quote.
     *
     * @param Model   $quote
     *
     * @return Response
     */
    public function destroy(QuoteRequest $request, Quote $quote)
    {
        try {

            $quote->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('service::quote.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('service/quote/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('service/quote/' . $quote->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple quote.
     *
     * @param Model   $quote
     *
     * @return Response
     */
    public function delete(QuoteRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('service::quote.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('service/quote'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/service/quote'))
                ->redirect();
        }

    }

    /**
     * Restore deleted quotes.
     *
     * @param Model   $quote
     *
     * @return Response
     */
    public function restore(QuoteRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('service::quote.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/service/quote'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/service/quote/'))
                ->redirect();
        }

    }

}
