<?php

namespace B2buy\Service\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use B2buy\Service\Http\Requests\SubscriptionRequest;
use B2buy\Service\Interfaces\SubscriptionRepositoryInterface;
use B2buy\Service\Models\Subscription;

/**
 * Resource controller class for subscription.
 */
class SubscriptionResourceController extends BaseController
{

    /**
     * Initialize subscription resource controller.
     *
     * @param type SubscriptionRepositoryInterface $subscription
     *
     * @return null
     */
    public function __construct(SubscriptionRepositoryInterface $subscription)
    {
        parent::__construct();
        $this->repository = $subscription;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\B2buy\Service\Repositories\Criteria\SubscriptionResourceCriteria::class);
    }

    /**
     * Display a list of subscription.
     *
     * @return Response
     */
    public function index(SubscriptionRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\B2buy\Service\Repositories\Presenter\SubscriptionPresenter::class)
                ->$function();
        }

        $subscriptions = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('service::subscription.names'))
            ->view('service::subscription.index', true)
            ->data(compact('subscriptions', 'view'))
            ->output();
    }

    /**
     * Display subscription.
     *
     * @param Request $request
     * @param Model   $subscription
     *
     * @return Response
     */
    public function show(SubscriptionRequest $request, Subscription $subscription)
    {

        if ($subscription->exists) {
            $view = 'service::subscription.show';
        } else {
            $view = 'service::subscription.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('service::subscription.name'))
            ->data(compact('subscription'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new subscription.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SubscriptionRequest $request)
    {

        $subscription = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('service::subscription.name')) 
            ->view('service::subscription.create', true) 
            ->data(compact('subscription'))
            ->output();
    }

    /**
     * Create new subscription.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SubscriptionRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $subscription                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('service::subscription.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('service/subscription/' . $subscription->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/service/subscription'))
                ->redirect();
        }

    }

    /**
     * Show subscription for editing.
     *
     * @param Request $request
     * @param Model   $subscription
     *
     * @return Response
     */
    public function edit(SubscriptionRequest $request, Subscription $subscription)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('service::subscription.name'))
            ->view('service::subscription.edit', true)
            ->data(compact('subscription'))
            ->output();
    }

    /**
     * Update the subscription.
     *
     * @param Request $request
     * @param Model   $subscription
     *
     * @return Response
     */
    public function update(SubscriptionRequest $request, Subscription $subscription)
    {
        try {
            $attributes = $request->all();

            $subscription->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('service::subscription.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('service/subscription/' . $subscription->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('service/subscription/' . $subscription->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the subscription.
     *
     * @param Model   $subscription
     *
     * @return Response
     */
    public function destroy(SubscriptionRequest $request, Subscription $subscription)
    {
        try {

            $subscription->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('service::subscription.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('service/subscription/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('service/subscription/' . $subscription->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple subscription.
     *
     * @param Model   $subscription
     *
     * @return Response
     */
    public function delete(SubscriptionRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('service::subscription.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('service/subscription'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/service/subscription'))
                ->redirect();
        }

    }

    /**
     * Restore deleted subscriptions.
     *
     * @param Model   $subscription
     *
     * @return Response
     */
    public function restore(SubscriptionRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('service::subscription.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/service/subscription'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/service/subscription/'))
                ->redirect();
        }

    }

}
